#define STRICT
#include <windows.h>

#include "resource.h"

static BOOL InitApp(HINSTANCE hInstance);
static BOOL InitInstance(HINSTANCE hInstance, int nCmdShow);
static LRESULT CALLBACK MainWndProc(HWND hWnd, UINT msg, WPARAM wp, LPARAM lp);
static BOOL PasswdWindowView(HWND hWnd, LPARAM lp);

static const char *szClassName = "PasswordView";
HINSTANCE g_hInstance;


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevinst, LPSTR lpCmdLine, int nCmdShow)
{
	MSG msg;
	
	if (!InitApp(hInstance))
		return FALSE;

	if (!InitInstance(hInstance, nCmdShow)) 
		return FALSE;

	while (GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return msg.wParam;
}


static BOOL InitApp(HINSTANCE hInstance)
{
	WNDCLASSEX wc;

	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc = MainWndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = LoadIcon(NULL, IDI_ASTERISK);
	wc.hCursor = NULL;
	wc.hbrBackground = (HBRUSH) GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = (LPCSTR) szClassName;
	wc.hIconSm = LoadIcon(NULL, IDI_APPLICATION);

	return RegisterClassEx(&wc);
}


static BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND hWnd;

	hWnd = CreateWindowEx(
		WS_EX_TOPMOST,
		szClassName,
		szClassName,
		WS_SYSMENU | WS_CAPTION,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		160,
		100,
		NULL,
		NULL,
		hInstance,
		NULL
	);

	if (!hWnd)
		return FALSE;

	g_hInstance = hInstance;

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

	return TRUE;
}


LRESULT CALLBACK MainWndProc(HWND hWnd, UINT msg, WPARAM wp, LPARAM lp)
{
	static BOOL bCap = FALSE;

	switch (msg) {
	case WM_PAINT:
		if (!bCap) {
			HDC hdc, hdc_mem;
			PAINTSTRUCT ps;
			HBITMAP hBit;
			BITMAP bmp_info;
			static int wx, wy;
			RECT rc;

			GetClientRect(hWnd, &rc);
			hdc = BeginPaint(hWnd, &ps);
			hBit = LoadBitmap(g_hInstance, MAKEINTRESOURCE(MOVE_BMP));
			GetObject(hBit, sizeof(BITMAP), &bmp_info);
			wx = bmp_info.bmWidth;
			wy = bmp_info.bmHeight;
			hdc_mem = CreateCompatibleDC(hdc);
			SelectObject(hdc_mem, hBit);
			BitBlt(hdc, (rc.right - wx) / 2, (rc.bottom-wy) / 2, wx, wy, hdc_mem, 0, 0, SRCCOPY);
			DeleteDC(hdc_mem);
			DeleteObject(hBit);
			EndPaint(hWnd, &ps);
		}

		break;

	case WM_LBUTTONDOWN:
		if (!bCap) {
			bCap = TRUE;
			SetCapture(hWnd);
			SetCursor(LoadCursor(g_hInstance, MAKEINTRESOURCE(MOVE_CURSOR)));
			InvalidateRect(hWnd, NULL, TRUE);
		}
		break;

	case WM_MOUSEMOVE:
		if (bCap)
			SetCursor(LoadCursor(g_hInstance, MAKEINTRESOURCE(MOVE_CURSOR)));
		else
			SetCursor(LoadCursor(NULL, IDC_ARROW));

		break;

	case WM_LBUTTONUP:
		ReleaseCapture();
		bCap = FALSE;
		SetCursor(LoadCursor(NULL, IDC_ARROW));

		if (!PasswdWindowView(hWnd, lp))
			MessageBox(hWnd, "s܂", NULL, MB_OK | MB_ICONWARNING);

		InvalidateRect(hWnd, NULL, TRUE);
		break;

	case WM_CREATE:
		SetCursor(LoadCursor(NULL, IDC_ARROW));
		break;

	case WM_CLOSE:
		DestroyWindow(hWnd);
		break;

	case WM_DESTROY:
		PostQuitMessage(0);
		break;

	default:
		break;
	}

	return DefWindowProc(hWnd, msg, wp, lp);
}


BOOL PasswdWindowView(HWND hWnd, LPARAM lp)
{
	POINTS pts;
	POINT pt;
	HWND hTarget;

	pts = MAKEPOINTS(lp);

	pt.x = pts.x;
	pt.y = pts.y;

	ClientToScreen(hWnd, &pt);

	hTarget = WindowFromPoint(pt);

	if (!hTarget)
		return FALSE;

	/* GetClassName of "Edit" */

	PostMessage(hTarget, EM_SETPASSWORDCHAR, 0, 0);
	InvalidateRect(hTarget, NULL, TRUE);

	return TRUE;
}



